/**
 * AlertDetailsController is a module that loads data and handles event logic called from {@link model.AlertDetailsModel}
 * Alert data is retrieved from {@link sso.sna.AlertsData}
 * @class controller.AlertDetailsController loads alert details data and renders the view using {@link model.AlertDetailsModel}
 */
define([
    'dojo/promise/all',
    'dojo/Deferred',
    '../util/ErrorUtil',
    '../util/I18nUtil',
    'sna/util/ContextPreferenceUtil',
    '../util/PluginUtil',
    'dojo/_base/lang',
    '../model/viewactions/ChartHelper',
    'dojo/_base/declare',
    'dojox/widget/Standby',
    '../util/TableUtil',
    '../data/AlertDetailsData',
    '../model/AlertDetailsModel',
    '../model/Locator',
    'dojox/lang/functional'
], function(all, Deferred, ErrorUtil, I18nUtil, ContextPreferenceUtil, PluginUtil, lang, ChartHelper, declare, Standby, tableUtil, alertDetailsData, AlertDetailsModel, locator, functional) {
    "use strict";

    var model;
    
    /**
     * executed when charts and alert transactions have finished loading
     * @param data a map representing the chart data
     * @param pluginDataMap a map of plugin data to use
     * @param groups a map of groups that go in the charts tab
     */
    var _onChartsLoaded = function(data, pluginDataMap, groups, plugins, actionableEntity) {
        var that = this;
        functional.forIn(groups, function(group) {
            group.filterFunction = that.getFilterQueryFunction();
        });

        ChartHelper.createChartObjectsFromData(data, pluginDataMap, groups, plugins, function (panelsWithCharts) {
            model.renderCharts(panelsWithCharts, actionableEntity);
            that.maximizedChartsController.start(panelsWithCharts);
        });
    };

    /**
     *
     * @param alertTransactions
     * @param tableEntriesMap
     * @param vertical
     * @param infoContentPaneStandby
     * @private
     */
    var _onAlertTransactionsLoaded = function(alertTransactions, tableEntriesMap, vertical, infoContentPaneStandby) {
        if(typeof(alertTransactions.error) !== 'undefined') {
            alertTransactions = ErrorUtil.createErrorTable(alertTransactions.error, 'error_table', {}, {optional: { tableDisplayName: I18nUtil.get('alert_details_alert_transactions_error')}});
        }

        tableEntriesMap = tableUtil.buildTableEntriesMap(tableEntriesMap, alertTransactions.tables);
        infoContentPaneStandby.destroyRecursive();

        model.buildAndRenderLowerRegionTables(tableEntriesMap, vertical);

        // start facets for alert transaction data
        // note: as of 1/7/2013, we don't have 'real' facets in the alert details view
        // instead, we just have the global search to act as a facet.
        // it's a bit magic how global search is enabled .. it's because the filter function (see FacetController.getFilterQueryFunction)
        // is set on every table that's rendered.
        // this assignment is done in AlertDetailsModel.buildAndRenderLowerRegionTables
        // also facets are rendered in this addFacets function call
        this.alertDetailsFacetController.addFacets({});

        // there was a bug where if you switched to the SNA tab before the details tab was done rendering
        // it would create two global search textboxes. This resolves that
        if(!model.detailsTabIsSelected()) {
            this.alertDetailsFacetController.hideGlobalSearchFacet();
        }
    };
    
    return declare([], {
        /**
         * @constructor creates a new AlertDetailsController
         * @param args object of the form
         * <pre><code>
         *  {
         *      dispositionController,
         *      networkViewController
         *  }
         * </pre></code>
         */
        constructor: function( args ) {
            this.dispositionController = args.dispositionController;
            this.networkController = args.networkController;
            this.maximizedChartsController = args.maximizedChartsController;
            this.alertDetailsFacetController = args.alertDetailsFacetController;
        },

        /**
         *  this function makes three calls to data using {@link sso.sna.AlertsData}:
         *  - sub alerts (also called related alerts)
         *  - alert transactions (other tabular data)
         *  - charts (data displayed in charts tab)
         *
         *  it does so asynchronously and then passes data to the @{link sso.sna.AlertDetailsModel} for rendering
         *
         *  @param entity the actionable entity to retrieve details for, which is an object having at least:
         *{
         *	actionableEntityType, (string)
         *	actionableEntityId (string)
         *}
         *    @param vertical (string) the analytic domain that the entity is from
         *    @private
         */
        _load: function( entity, vertical ) {
            var that = this;

            var visualContentPaneStandby = new Standby({
                target: "visualcontentpane",
                image: 'src/dijit/applicationThemes/corporate/images/sasIcons/sasdark/other_png/ProgressIndicatorSpinner.gif'
            });
            document.body.appendChild(visualContentPaneStandby.domNode);
            visualContentPaneStandby.startup();
            visualContentPaneStandby.show();

            var tabularDataTabContentPaneStandby = new Standby({
                target: "tabulardatatab",
                image: 'src/dijit/applicationThemes/corporate/images/sasIcons/sasdark/other_png/ProgressIndicatorSpinner.gif'
            });
            document.body.appendChild(tabularDataTabContentPaneStandby.domNode);
            tabularDataTabContentPaneStandby.startup();
            tabularDataTabContentPaneStandby.show();

            var infoContentPaneStandby = new Standby({
                target: "infocontentpane",
                image: 'src/dijit/applicationThemes/corporate/images/sasIcons/sasdark/other_png/ProgressIndicatorSpinner.gif'
            });
            document.body.appendChild(infoContentPaneStandby.domNode);
            infoContentPaneStandby.startup();
            infoContentPaneStandby.show();

            var numLoaded = 0;
            var alertTransactions;
            var subAlerts;
            var tableEntriesMap = {};
            var chartsLoaded = false;

            var headerId = entity.headerId;
            if(!entity.headerId) {
                headerId = 0;
            }

            // since data is loaded asynchronously, this will be passed to plugins and populated as the data loads
            var pluginDataMap = {
                actionableEntity: entity
            };

            var chartsPromise = new Deferred();
            var processCharts = function(data) {
                var chartData = data;

                if(typeof(chartData.error) !== 'undefined') {
                    chartData = ErrorUtil.createErrorChart(chartData.error, {}, {});
                }

                pluginDataMap.charts = data;

                lang.hitch(that, _onChartsLoaded, chartData, pluginDataMap, alertTransactions.groups, alertTransactions.plugins, entity)();
                chartsLoaded = true;
                tabularDataTabContentPaneStandby.destroyRecursive();
                visualContentPaneStandby.destroyRecursive();
            };
            alertDetailsData.getCharts(vertical, 'en_US', entity, function(data) {
                chartsPromise.resolve(data);
            });

            var subAlertsPromise = new Deferred();
            var processSubAlerts = function(data) {
                var subAlerts = data;
                pluginDataMap.subAlerts = data;

                var errorOccurred = typeof(data.error) !== 'undefined';
                if(errorOccurred) {
                    console.warn('AlertDetailsController.load: no subalerts were loaded ', data);
                    subAlerts = ErrorUtil.createErrorTable(data.error, 'MEASURES', {}, {optional: { tableDisplayName: I18nUtil.get('alert_details_related_alerts_error')}});
                }

                var subAlertsAction = model.subAlertsLoaded(subAlerts);
                // per Sharon, users have not been allowed to rename the related alerts table, so I took out that
                // section.
                var displayName = I18nUtil.get('alert_details_related_alerts');

                var showRelatedAlertsFlag = ContextPreferenceUtil.getPreference(ContextPreferenceUtil.SHOW_RELATED_ALERTS);
                if(typeof(showRelatedAlertsFlag) === 'undefined') {
                    showRelatedAlertsFlag = true;
                } else {
                    showRelatedAlertsFlag = (showRelatedAlertsFlag === 'y');
                }

                if(showRelatedAlertsFlag) {
                    tableEntriesMap.Related_Alerts = {
                        values:subAlerts.tables.MEASURES.values,
                        columns:subAlerts.tables.MEASURES.columns,
                        seriesId:'subalerts',
                        errorTable: subAlerts.tables.MEASURES.errorTable,
                        facets:subAlerts.tables.MEASURES.facets,
                        optional:{
                            tableDisplayName: displayName
                        },
                        onRowClick: function(row) {
                            // extra function call to preserve subAlertsAction as "this" in rowClicked
                            subAlertsAction.rowClicked(row);
                        },
                        onRowCheck: function(row) {
                            subAlertsAction.rowClicked(row);
                        },
                        otherOptions: {
                            enableSelection: true,
                            selectionMode: 'single',
                            selectFirst: true
                        }
                    };
                }
            };
            alertDetailsData.getSubAlerts(vertical, 'en_US', entity, function(data) {
                subAlertsPromise.resolve(data);
            });

            var alertTransactionsPromise = new Deferred();
            var processAlertTransactions = function(data) {
                alertTransactions = data;
                pluginDataMap.alertTransactions = data;
            };
            alertDetailsData.getAlertTransactions(vertical, 'en_US', entity, entity.entityId, headerId, function(data) {
                alertTransactionsPromise.resolve(data);
            });

            subAlertsPromise.then(function(results) {
                processSubAlerts(results);
            });

            all([alertTransactionsPromise, chartsPromise, subAlertsPromise]).then(function(results) {
                processAlertTransactions(results[0]);
                processCharts(results[1]);
                // already processed subalerts

                that._runHiddenPlugins(pluginDataMap, alertTransactions.hiddenPlugins);
                lang.hitch(that, _onAlertTransactionsLoaded, alertTransactions, tableEntriesMap, vertical, infoContentPaneStandby)();

                // wait until all three calls are complete to re-enable the back to alerts button
                var backButton = dijit.byId('backtoalertsbutton');
                var refreshButton = dijit.byId('refreshbutton');

                if(backButton) {
                    backButton.set('disabled', false);
                }
                if(refreshButton) {
                    refreshButton.set('disabled', false);
                }
            });
        },

        /**
         * executes hidden plugins in the alert details view
         * @param pluginDataMap map of data to pass
         * @param plugins see {@link util.PluginUtil} for the structure
         */
        _runHiddenPlugins: function(pluginDataMap, plugins) {
        	PluginUtil.createPluginsFromMap(plugins, pluginDataMap);
        },

        /**
         * creates the {@link model.AlertDetailsModel} and loads the data
         * @param entity the actionable entity to load data for
         * @param vertical the analytic context
         */
        start: function (entity, vertical) {
            this.alertDetailsFacetController.start(null, locator.locate().globalsearch());

            model = new AlertDetailsModel({
                d_context:locator.locate().maincontent(),
                controller: this,
                vertical: vertical
            });
            
            var showNetworkFlag = ContextPreferenceUtil.getPreference(ContextPreferenceUtil.SHOW_NETWORK_VIEW);
            if(typeof(showNetworkFlag) === 'undefined' || showNetworkFlag === 'y') {
                showNetworkFlag = true;
            }
            else {
                showNetworkFlag = false;
            }

            model.loading(entity, vertical, showNetworkFlag);
            this._load(entity, vertical);
        },

        /**
         * returns true if the alert details view ha been loaded
         * @return {Boolean}
         */
        isLoaded:function () {
            var isLoaded = false;

            if (model) {
                isLoaded = model.isLoaded();
            }

            return isLoaded;
        },

        /**
         * get tables and charts for printing
         * @return {*}
         */
        getPrintData: function() {
            return model.getPrintData(); 
        },

        getEntity: function() {
            return model.getEntity(); 
        },

        cleanup: function() {
            if( model ) {
                model.cleanupView();
            }

            this.alertDetailsFacetController.destroy();
            this.networkController.networkFacetController.destroy();
        },

        /**
         * updates the list of possible disposition actions
         * @param entitiesArray the array of actionable entities that have been selected
         * @param view the current view ('alertdetails')
         */
        updateDispositionItemsList: function(entitiesArray, view) {
            this.dispositionController.updateDispositionItemsList(entitiesArray, view);
        },

        /**
         * loads the network view controller
         * @param vertical the analytic context
         * @param e the actionable entity
         */
        startNetworkViewController: function( vertical, actionableEntity ) {
            this.networkController.start(vertical, actionableEntity);
        },
        
        showMaximizedCharts: function() {
        	this.maximizedChartsController.show();
        },

        refreshDataGrid: function() {
            model.refreshDataGrid();
        },

        getFilterQueryFunction: function () {
            return this.alertDetailsFacetController.getFilterQueryFunction();
        },

        isDGridReady: function() {
            return model.isDGridReady();
        }
    }); // end of return
});