"use strict";

/**
* AlertsViewController is a module responsible for retrieving alert data, handling complex events from {@link sso.sna.AlertsViewModel},
* and switching to other controllers. Data is retrieved by interacting with {@link sso.sna.AlertsData}
*
* @class controller.AlertsViewController Retrieves alert data and manipulates model to display alerts
*
*/
define([
    'sna/model/widgets/LoadingSpinner/LoadingSpinner',
    'dijit/registry',
    '../util/ErrorUtil',
	'../util/ContextPreferenceUtil',
	'../util/PluginUtil',
    'sna/util/OverlayUtil',
    'dojo/_base/declare',
    'dojo/dom-style',
    'sna/model/Locator',
    'sna/data/AlertsViewData',
    'sna/model/AlertsViewModel'
], function(LoadingSpinner, registry, ErrorUtil, ContextPreferenceUtil, PluginUtil, overlayUtil, declare, domStyle, locator, alertsViewData, AlertsViewModel ) {
	return declare([], {
		constructor: function( args ) {
		    this.viewSwitcherController = args.viewSwitcherController;
			this.alertsListFacetController = args.alertsListFacetController;
			this.alertDetailsController = args.alertDetailsController; 
			this.dispositionController = args.dispositionController; 
			this.externalAppController = args.externalAppController;
		},
		
		/**
		 * entry point for AlertsViewController. Starts the {@link sso.sna.AlertsViewModel} default loading state, and then
		 * retrieves the alerts data
		 *
		 * @param vertical the analytic domain to retrieve alerts from
		 * @param submissions advanced search parameters to send to server, or null
		 */
		start: function(vertical, submissions, fetchEntities, useCached ) {
            console.info('AlertsViewController: Are we using cached alerts data? ', typeof(useCached) !== 'undefined');

			/* remove previous view's data */
			if( !this.model ) {
				this.model = new AlertsViewModel({
					controller: this,
					d_context: locator.locate().maincontent()
				});
			}
			
			this.alertsListFacetController.start(locator.locate().alertsfacetcontent(), locator.locate().globalsearch());
			this.fetchAlertedEntities({}, vertical, submissions, fetchEntities, useCached);
		},

		/**
		 * switches to the alert details view when an alert is clicked
		 * @param entity the actionable entity to load details for. Must have at least the structure:
		 * <pre><code>
		 *{
		 *	actionableEntityId,
		 *	actionableEntityType
		 *}
		 * </code></pre>
		 * @param vertical the analytic domain to retrieve alert details for
         * @param submissions any search criteria that were used
         * @method
		 */
		alertClicked: function(entity, vertical, submissions ) {
			console.log('leaving alerts view'); 
			this.viewSwitcherController.switchToDetailsPage(entity, vertical, submissions);
		},
		
		cleanup: function() {
			if(this.model) {
			    this.model.cleanupView();
			}
			
			this.alertsListFacetController.destroy();
		},

		/**
		 * gets data from the {@link model.AlertsViewModel} that can be displayed on the print dialog
		 * @return an object that has the following structure:
		 * <pre><code>
		 *{
		 *	table name: {
		 *		valid html code representing a data table
		 *	}
		 *}
		 * </code></pre>
		 * @type object
         * @method
		 */
		getPrintData: function() {
			return this.model.getPrintData();
		},

		dispositionCompleted: function() {
            console.log('AlertsViewController.dispositionCompleted');
            this.model.dispositionCompleted();
		},

		/**
		 * loads hidden plugins for the alerts view
		 *
		 * see {@link sso.sna.util.PluginUtil} for parameter structure
		 */
		loadPlugins: function(plugins, data, vertical) {
            if(typeof(plugins) === 'undefined' || typeof(data) === 'undefined' || typeof(vertical) === 'undefined') {
                console.warn('AlertsViewController.loadPlugins: a parameter was undefined', plugins, data, vertical);
            }

            PluginUtil.createPluginInstances(plugins, {alertsData: data}, vertical);
		},

        /**
         * gets an array of selected actionable entities
         */
        getSelectedAlerts: function() {
            if(this.model) {
                return this.model.getSelectedAlerts();
            }
            else {
                return {};
            }
        },

		getAdvancedSearchParameters: function() {
			if(this.model) {
				return this.model.getAdvancedSearchParameters();
			}
			else {
				return {};
			}
		},
		
		refreshDataGrid: function() {
			this.model.refreshDataGrid();
		},
		
		getColumnIndexFromColumnIdentifier: function( columnIdentifier ) {
			return this.model.getColumnIndexFromColumnIdentifier(columnIdentifier);
		},
		
		isDGridReady: function() {
			return this.model.isDGridReady();
		},

        /**
         * calls the model to remove all rows that are checked on the datagrid
         */
        removeCheckedRows: function() {
            console.log('AlertsViewController.removeCheckedRows');
            this.model.removeCheckedRows();
            this.model.refreshDataGrid();
        },

        /**
         * calls the model to remove rows that match any id in the passed in array of ids
         * @param entityIds an array of objects with the minimal structure:
         * <pre><code>
         * {
         *      actionableEntityId
         * }
         * </pre></code>
         */
        removeRowById: function(entityIds) {

        },

        /**
         * retrieves the list of alerts and then starts rendering the {@link sso.sna.AlertsViewModel}
         *
         * @param $context the top-level container that the alerts view should render to
         * @param vertical the analytic domain to retrieve data from
         * @param submissions
         * @param fetchEntities if this is false, no alerts data will be retrieved
         * @param useCached if true, this will use cached alerts data
         */
        fetchAlertedEntities:function ($context, vertical, submissions, fetchEntities, useCached) {
            var that = this;
            
            if(fetchEntities || typeof(fetchEntities) === 'undefined') {            
                var completedLoadingFlag = false;
                if( !useCached ) {
                    new LoadingSpinner().target(registry.byId("container")).until(function() { return completedLoadingFlag; }).startSpinners(function() {});
                }
	            alertsViewData.getAlertedEntities(vertical, submissions, 'en_US', function (data) {
                    // destroy the datagrid only after fetching fresh data
                    that.model.destroyDataGrid();

                    if(that.model.isDGridReady()) {
                        console.warn('AlertsViewController: AlertsViewModel did not have d_datagrid reset to undefined, probably due to improper cleanup');
                    }

	            	that.alertsListFacetController.addFacets(data.facets);

                    var errorOccurred = typeof(data.error) !== 'undefined';
                    if(errorOccurred) {
                        console.warn('AlertsViewController.fetchAlertedEntities: alerted entities had no data ', data);
                        data = ErrorUtil.createErrorTable(data.error, 'MEASURES', {facets:[]});
                    }
                    
                    var showFacetPanel = ContextPreferenceUtil.getPreference(ContextPreferenceUtil.SHOW_ALERTS_FACET_PANEL);
                    if(typeof(showFacetPanel) === 'undefined') {
                        showFacetPanel = true;
                    }

                	that.model.render(vertical, data, submissions, showFacetPanel);
                    that.loadPlugins(data.plugins, data, vertical);
	                overlayUtil.hideOverlay();
                    completedLoadingFlag = true;
	                
	            }, useCached);
            }
            else {
            	console.warn('AlertsViewController.fetchAlertedEntities: not fetching any data');
            	
            	that.alertsListFacetController.addFacets({});
            	that.model.render(vertical, {}, submissions);
                overlayUtil.hideOverlay();
            }
            
        }, // end of fetchAlertedEntities

        getFilterQueryFunction:function () {
            return this.alertsListFacetController.getFilterQueryFunction();
        },

        updateDispositionItemsList:function (entities, view) {
            this.dispositionController.updateDispositionItemsList(entities, 'alerts');
        },
        
        startDispositionController:function (vertical, callback) {
            this.dispositionController.start(vertical, callback);
        },
        
        updateExternalAppsList:function(entities, view) {
        	this.externalAppController.updateExternalAppsList(entities, 'alerts');
        },

        startExternalAppController:function (callback) {
            this.externalAppController.start(callback);
        }
	}); // end of return
}); 
