/**
 * @class controller.DispositionController handles data and the model for dispositioning
 */
define([
    'sna/model/viewactions/ObjectModelUtil',
    'dojo/_base/array',
    'dojo/_base/declare',
    '../model/DispositionModel',
    '../data/DispositionData',
    'dojox/lang/functional'
], function(ObjectModelUtil, array, declare, DispositionModel, dispositionData, functional) {
    var POSSIBLE_DIALOG_ACTIONS = {
        NOOP: 0,
        UPDATE_AE: 1,
        REMOVE_AE: 2,
        FULL_REFRESH: 3
    };

    return declare([], {
        constructor:function (args) {
            this.alertDetailsController = args.alertDetailsController;
            this.alertsViewController = args.alertsViewController;
            this.viewSwitcherController = args.viewSwitcherController;
        },

        start:function (vertical, callback) {
            var that = this;

            dispositionData.getDispositionActions(vertical, 'sasdemo', function(data) {
                console.log('disposition data', data);
                
                that.model = new DispositionModel({
                    dispositionElements: data,
                    vertical: vertical,
                    controller: that
                });
                
                // whether the disposition toolbar button should be enabled/disabled
                if( data ) {
                	that.enable();
                }
                else {
                	that.disable();
                }

                if( callback ) {
                    callback();
                }
            });
        },

        /**
         * calls the model to update the disposition drop-down list
         *
         * @param selectedEntitiesArray an array of actionable entities selected for dispositioning
         * @param currentView the current view in the application
         */
        updateDispositionItemsList: function(selectedEntitiesArray, currentView) {
            this.model.updateDispositionList(selectedEntitiesArray, currentView);
        },

        /**
         * filters disposition actions by the current view
         *
         * @param currentView the current view in the application
         * @param dispositionElementsMap a map of disposition actions where the key is the action id
         * @return {Array} the array of disposition actions that are good for the passed in view
         */
        filterDispositionItemsByView: function(currentView, dispositionElementsMap) {
            var results = [];

            functional.forIn(dispositionElementsMap, function(value, key) {
                var regex;
                if(value.view !== '*') {
                    regex = new RegExp(value.view, 'i');
                }

                var entryMatches = false;

                if(value.view === '*' || regex.test(currentView)) {
                    entryMatches = true;
                }

                if(entryMatches) {
                    results.push(value);
                }
            });

            return results;
        },

        /**
         * filters the disposition elements map based on actionable entity properties
         *
         * @param selectedEntitiesArray the array of actionable entities
         * @param dispositionElementsMap the map of disposition actions where the key is the action id
         * @return (Array) the list of filtered disposition actions
         */
        filterDispositionItemArray: function(selectedEntitiesArray, dispositionElementsMap) {
            var results = [];

            functional.forIn(dispositionElementsMap, function(value, key) {
                var regex;
                if(value.entityType !== '*') {
                    regex = new RegExp(value.entityType, 'i');
                }

                var entryMatches = false;

                if(value.entityType === '*' ||  selectedEntitiesArray.length === 0 || regex.test(selectedEntitiesArray[0].actionableEntityType)) {
                    entryMatches = true;
                }

                if(!value.supportsMultiple && selectedEntitiesArray.length > 1) {
                    entryMatches = false;
                }

                if(selectedEntitiesArray.length === 0) {
                    entryMatches = false;
                }

                results.push({
                    value: value,
                    enabled: entryMatches
                });
            });

            return results;
        },

        dispositionActionClicked: function(vertical, dispositionKey, parametersMap) {
        	var that = this; 
            console.log('dispositioncontroller.dispositionActionClicked');

            dispositionData.getDispositionUI(vertical, dispositionKey, parametersMap, function(uiDataMap) {
                that.model.renderDispositionUI(uiDataMap, dispositionKey);
            });
        },

        populateParametersMap: function(parametersArray, selectedEntitiesArray) {
            var parametersMap = {};

            functional.forIn(parametersArray, function(value, key) {
                if(value.startsWith('entity.')) {
                    parametersMap[value] = selectedEntitiesArray[0][value.replace('entity.', '')];
                }
            });

            return parametersMap;
        },

        /**
         * submits disposition parameters to the mid-tier (and thus, the processAlertResponse STP) and retrieves the response
         *
         * @param vertical the analytic domain
         * @param actionableEntitiesArray list of actionable entities to send across
         * @param dispositionCriteria an array of form parameters, where each one has the structure:
         * <pre><code>
         * {
         *      queryType: (string) the tab that this element is in on the dialog
         *      queryParameter: the group that this element is in on the tab
         *      queryValue: the value of the parameter
         * }
         * </pre></code>
         * @param selectedPanel the selected action for dispositioning
         * @param callback callback to execute once a response is received
         */
        submitDispositionResponse: function(vertical, actionableEntitiesArray, dispositionCriteria, selectedPanel, callback) {
            console.log("DispositionController.submitDispositionResponse", vertical, actionableEntitiesArray, dispositionCriteria, selectedPanel, callback);

            // before submitting, safely unescape the actionable entity data
            array.forEach(actionableEntitiesArray, function(entity) {
                functional.forIn(entity, function(value, property) {
                    if(property === 'actionableEntityId') {
                        return;
                    }

                    if(typeof value === 'string') {
                        entity[property] = ObjectModelUtil.safeUnescapeValue(value);
                    }
                })
            });

            dispositionData.submitAction(vertical, selectedPanel, actionableEntitiesArray, dispositionCriteria, function(data) {
                if(callback) {
                    callback(data);
                }
            });
        },

        /**
         *
         * @param code
         * @param isDetailsView
         * @param selectedEntities
         * @param vertical
         */
        takeActionBasedOnResponse:  function(code, isDetailsView, selectedEntities, vertical) {
            console.log('DispositionController.takeActionBasedOnResonse', code, isDetailsView, selectedEntities, vertical);

            if(code === POSSIBLE_DIALOG_ACTIONS.REMOVE_AE) {
                if(!isDetailsView) {
                    this.alertsViewController.removeCheckedRows();
                }
                else {
                    this.viewSwitcherController.switchToAlertsListPage(true, vertical, this.alertsViewController.getAdvancedSearchParameters());
                    this.alertsViewController.removeRowById(selectedEntities);
                }
            }
            else if (code === POSSIBLE_DIALOG_ACTIONS.FULL_REFRESH) {
                if(!isDetailsView) {
                    this.viewSwitcherController.switchToAlertsListPage(true, vertical, this.alertsViewController.getAdvancedSearchParameters());
                }
                else {
                    this.viewSwitcherController.switchToAlertsListPage(true, vertical, this.alertsViewController.getAdvancedSearchParameters());
                }
            }
        },

        alertDetailsIsLoaded:function () {
            return this.alertDetailsController.isLoaded()
        },

        dispositionCompleted:function () {
            this.alertsViewController.dispositionCompleted();
        },

        disable:function () {
            this.model.disableButton();
        },

        enable:function () {
            this.model.enableButton();
        }
    });
});
