
/**
 * @class controller.AlertsListFacetController
 */
define([
    '../model/viewactions/ObjectModelUtil',
    '../util/ContextPreferenceUtil',
    'dojo/_base/array',
    'dojo/_base/lang',
    'dojox/lang/functional',
    'dojo/_base/declare',
    '../model/viewactions/FacetHelper',
    'dijit/layout/ContentPane',
    'dojo/aspect'
], function(ObjectModelUtil, ContextPreferenceUtil, array, lang, functional, declare, FacetHelper, ContentPane, aspect ) {
    return declare([], {
        /**
         * @param args object of the structure
         * <pre><code>
         *     userPreferenceKey: identifier used for facet user preferences
         * </pre></code>
         */
        constructor: function(args) {
            this._userPreferenceKey = args.userPreferenceKey;
            this.controller = args.controller;

            // a map of callback functions for grouping that will be passed to FacetHelper
            var that = this;
            this.facetEventMap = {
                onFacetChange: function() {
                    that.triggerGridRefresh();
                }
            };

            this.facets = {};
        },

        /**
         * Add facets that are prepared from the passed-in facet json data
         */
        addFacets: function( facetDataMap ) {
            if(!facetDataMap) {
                facetDataMap = {};
            }

            FacetHelper.addGlobalSearchFacet(facetDataMap);

            // get map of facet objects derived from the facet json
            var facetObjects = FacetHelper.prepareFacets(facetDataMap, this.facetEventMap, 'ALERTS');

            var that = this;
            // add the global search filter function
            facetObjects['global_search_facet'].customFilter = this.globalSearchFilterFunction;
            facetObjects['global_search_facet'].customRender = function(d_container, facet) {
                FacetHelper.globalSearchRenderer(that.globalSearch, d_container, facet);
            };

            // add these facets to this function's facets reference
            // (note: must use mixin, rather than regular assignment, to preserve reference)
            /* // this is commented out because we are resetting the facets map in the start function
             if( this.facets ) {
             array.forEach(Object.keys(this.facets), function( value ) {
             delete this.facets[value];
             });
             }
             */
            lang.mixin(this.facets, facetObjects);

            // render facets to the d_container
            FacetHelper.renderFacets(this.facets, this.d_container);
        },

        /**
         * Custom facet filter function that provides functionality for the global search textbox
         * @param facet the facet object this function is bound to
         * @param rowArray array representing an individual row of data
         * @private
         */
        globalSearchFilterFunction: function(facet, rowArray) {
            var searchSatisfied = false;
            var value = facet.getValue();
            var valueLowercase = value.toLowerCase();

            array.forEach(rowArray, function(elementArray) {
                var rowValue = ObjectModelUtil.getFormattedValue(elementArray);

                if(rowValue.toLowerCase().indexOf(valueLowercase) !== -1) {
                    searchSatisfied = true;
                    return; // exit loop
                }
            });

            return searchSatisfied;
        },

        /**
         * Trigger the alerts list dgrid to refresh
         */
        triggerGridRefresh: function() {
            this.controller.refreshDataGrid();
        },

        /**
         * determines whether the alerts view dgrid initialized and ready to perform actions on
         */
        isDGridReady: function() {
            return this.controller.isDGridReady();
        },

        /**
         * Get the column index with the passed-in facet id from the alerts view dgrid
         */
        getColumnIndexFromColumnIdentifier: function( facetid ) {
            return this.controller.getColumnIndexFromColumnIdentifier(facetid);
        },

        /**
         * Returns a function that can be passed to the dgrid to use when filtering out rows.
         * It is determined here, because it uses the facets that are in closure scope
         */
        getFilterQueryFunction: function() {
            var that = this;

            return function( rowArray, iCurrentIteration, allRowsArray ) {
                // determines whether the row should stay or be hidden by dgrid
                var keepRow = true;

                // don't perform the filtering if the dgrid isn't initialized yet
                if (that.isDGridReady() === true) {
                    functional.forIn(that.facets, function (facet, index) {
                        var facetid = facet.name;

                        if(facet.customFilter) {
                            // used only for global search facet at the moment because it needs to search on all columns
                            if(facet.customFilter(facet, rowArray) === false) {
                                keepRow = false;
                                return false; // break out of loop
                            }
                        }
                        else {
                            // get the index of the column associated with this facet
                            var columnIndex = that.getColumnIndexFromColumnIdentifier(facetid);

                            if(columnIndex === -1) {
                                return; // continue, don't filter because the column is invalid
                            }
                            else {
                                // individual value from dgrid to be compared with the currently iterated facet
                                var dgridCellValue = ObjectModelUtil.getRawValue(rowArray[columnIndex]);

                                // use the filter logic that each facet has
                                if (facet.computeFilterLogic(dgridCellValue) === false) {
                                    keepRow = false;
                                    return false; // break out of loop (functional.forIn)
                                }
                            }
                        }
                    });
                }

                return keepRow;
            };
        },

        /**
         * destroy the facets container
         */
        destroy: function() {
            if( this.d_container ) {
                this.d_container.destroyDescendants();
            }

            FacetHelper.destroyFacets(this.facets);
        }
    });
});
