/**
 * @class controller.SessionMonitorController maintains a session with the server if the user desires
 * Controls warning the user when they are going to be logged out
 */
define([
    '../data/SessionMonitorData',
    'dojo/number',
    '../util/ContextPreferenceUtil',
    '../model/widgets/SessionMonitorDialog/SessionMonitorModel',
    'dojo/_base/lang',
    'dojo/request',
    'dojo/aspect',
    'dojo/_base/declare'
], function(sessionMonitorData, number, contextPreferenceUtil, SessionMonitorModel, lang, request, aspect, declare ) {
    var DEFAULT_TIMEOUT_WARNING_INTERVAL = 10 * 1000 * 60;

    return declare([], {
        constructor: function( args ) {
            this.userController = args.userController;

            this.state = {
                timer: 0,
                warned: false,
                pollInterval: 1000 * 60 // minutes
            };
        },

        logoff: function() {
            window.onbeforeunload = function() {};
            this.userController.logoff();
        },

        /**
         *
         * @param context this since the function is called by setTimeout
         * @private
         */
        _timingPollFunction: function(context) {
            context.state.timer += context.state.pollInterval;

            if(context.state.timer >= context.state.logoffTime) {
                context.logoff();
            }
            else if(context.state.timer >= context.state.userTimeoutWarning) {
                context._showTimeoutWarning();
            }

            setTimeout(function() { context._timingPollFunction(context); }, context.state.pollInterval);
        },

        _showTimeoutWarning: function() {
            if(this.state.warned) {
                return;
            }

            if(typeof(this.model) !== 'undefined') {
                return;
            }

            this.model = new SessionMonitorModel({
                controller: this
            }).show();

            this.state.warned = true;
        },

        /**
         * gets the amount of time that the user has been inactive for
         * @return {Number}
         */
        getInactiveTime: function() {
            return Math.round(this.state.timer / 1000 / 60);
        },

        /**
         * makes a server request to keep the session alive
         */
        resetSessionStatus: function() {
            this.state.warned = false;
            this.state.timer = 0;

            if(this.model) {
                this.model = undefined;
            }

            sessionMonitorData.keepSessionAlive();
        },

        /**
         * starts the intercepting of requests and monitoring of the session
         */
        start: function() {
            if(this.state.postAspect) {
                console.info('SessionMonitorController: removed previous interceptor functions');
                this.state.postAspect.remove();
            }

            if(this.state.getAspect) {
                this.state.getAspect.remove();
            }

            var warningTime = DEFAULT_TIMEOUT_WARNING_INTERVAL;
            var warningTimePreference = contextPreferenceUtil.getPreference('logout_warning_time');

            if(typeof(warningTimePreference) !== 'undefined') {
                warningTime = number.parse(warningTimePreference) * 1000 * 60; // user response is expected in minutes
            }

            this.state.userTimeoutWarning = warningTime;
            console.info('SessionMonitorController: timeout warning assigned a value of %0'.tokenize(warningTime));

            var that = this;

            // intercept every get and post request to the server and reset the timer on success responses
            var interceptorFunction = function(deferred) {
                deferred.then(function(response) {
                    that.state.timer = 0;
                },
                function(error) {
                    // don't reset the timer
                });

                return deferred;
            };

            this.state.logoffTime = this.userController.getTimeout() * 1000; // this is always coming from the server in seconds
            console.info('SessionMonitorController assigned a logoff time of %0'.tokenize(this.state.logoffTime));

            this.state.postAspect = aspect.after(request, 'post', interceptorFunction);
            this.state.getAspect = aspect.after(request, 'get', interceptorFunction);

            console.info('SessionMonitorController is intercepting all request get and post calls');

            setTimeout(function() { that._timingPollFunction(that); }, this.state.pollInterval);
        }
    });
});
