/**
 * @class model.AdvancedSearchModel creates advanced search dialog and handles events from it
 */
define([
    'dojo/dom-class',
    'sna/model/widgets/LoadingSpinner/LoadingSpinner',
    'sna/util/I18nUtil',
    '../util/UserPreferenceUtil',
    'dojo/html',
    'dojo/_base/array',
    'dijit/MenuItem',
    'dojo/number',
    'dojox/lang/functional',
    'dojo/_base/lang',
    './viewactions/DynamicUIUtil',
    'dojo/query',
    'dojo/on',
    './viewactions/DynamicUIRenderer',
    'dijit/registry',
    'dijit/Dialog',
    'dojo/text!src/sna/resources/html/templates/advancedsearch.html',
    'dojo/_base/declare'
], function( domClass, LoadingSpinner, I18nUtil, UserPreferenceUtil, html, array, MenuItem, number, functional, lang, dynamicUIUtil, query, on, DynamicUIRenderer,
            registry, Dialog, template, declare ) {
	var preferenceKey = 'advancedsearch';

    var savePreferences = function () {
    	UserPreferenceUtil.save(preferenceKey, this.renderer.enumerateValues());
    };

    var applyPreferences = function () {
        var that = this;
        UserPreferenceUtil.retrieve(preferenceKey, function (data) {
            that.renderer.setValues(data);
        });
    };

    var resetValues = function () {
        this.renderer.reset();
    };

    var getToolbarButton = function () {
        return registry.byId('advancedsearchbutton');
    };

    return declare([], {
        /**
         * @constructor creates a new AdvancedSearchModel
         * @param args object of the form
         * <pre><code>
         *     {
         *          controller: the advanced search controller that controls this model
         *     }
         * </pre></code>
         */
        constructor:function (args) {
            this.controller = args.controller;
        },

        /**
         * renders the advanced search dialog and wires up events
         * @param vertical the analytic context
         * @param uiDataMap object that represents the UI, see {@link model.viewactions.DynamicUIRenderer} for more details
         * @param hideCloseButton if false, cancel event will be ignored
         */
        render: function(vertical, uiDataMap, hideCloseButton) {
            var ui = uiDataMap.ui;
            var that = this;
            console.log('AdvancedSearchModel.renderDispositionUI', vertical, ui);

            // load disposition dialog
            var d_dialog = new Dialog({
                title: uiDataMap.properties.title,
                content: template,
                style: 'width: 600px; height:500px;',
                _onChildChange: function() {
                    // TODO Dialog uses _FormMixin, which for some reason tries to pick up checkbox values
                    // and throws an error. So, this function is overwritten for now.
                }
            });
            domClass.add(d_dialog.domNode, "advancedsearchdialog");
            d_dialog.startup();

            if(hideCloseButton) {
                d_dialog.closeButtonNode.style.display = 'none';
            }

            // build the dynamic UI
            var d_content = registry.byNode(query('.content', d_dialog.domNode)[0]);
            this.renderer = new DynamicUIRenderer({
                d_context: d_content
            });
            this.renderer.render(ui);

            // show dialog and wire up events
            d_dialog.show();
            on(d_dialog, 'hide', function() {
                d_dialog.destroyDescendants();
                d_dialog.destroy();
            });
            
            on(d_dialog, 'cancel', function() {
            	if(hideCloseButton) {
            		// for this case, current behavior in the flash version is to simply
            		// not submit any alerts
            		that.controller.submissionFunction({}, vertical, {}, false);
            	}
            }); 
            
            // hook up search button
            var d_searchButton = registry.byId('advancedsearch-search');
            d_searchButton.setLabel(uiDataMap.properties.buttonLabel);
            on(d_searchButton, 'click', function () {
                lang.hitch(that, savePreferences)();
                that.startSearch(vertical, true);
            });
            
            // set up the max rows select element
            var d_maxRowsSelect = registry.byId('advancedsearch-maxrows');
            d_maxRowsSelect.constraints = {
                max: uiDataMap.properties.maxRowsMax,
                min: uiDataMap.properties.maxRowsMin
            };
            d_maxRowsSelect.set('value', uiDataMap.properties.maxRowsValue);
            on(d_maxRowsSelect, 'change', function() {
            	that.enforceConstraints(d_maxRowsSelect); 
            }); 
            d_maxRowsSelect.smallDelta = uiDataMap.properties.maxRowsStep || 1;


            this._d_maxRowsSelect = d_maxRowsSelect;

            // set up the label
            var maxRowsLabel = query('#advancedsearch-maxrowslabel', d_dialog.domNode)[0];
            html.set(maxRowsLabel, uiDataMap.properties.maxRowsLabel);
            // localize the advanced search restore label as well
            registry.byId('advancedsearch-restore').set('label', I18nUtil.get('preferences_restore'));

            this._d_dialog = d_dialog;
            this._groups = this.renderer.getGroups();
            this._data = uiDataMap;
            
            lang.hitch(that, applyPreferences)();
        },

        enforceConstraints: function(d_spinner) {
        	 var value =  d_spinner.value;
             if(value < d_spinner.constraints.min) {
             	value = d_spinner.constraints.min; 
             }
             else if (value > d_spinner.constraints.max) {
             	value = d_spinner.constraints.max; 
             }
             
             d_spinner.setValue(value); 
        },
        
        /**
         * called when the user clicks "Search." Gathers the form parameters and submits them to the controller
         * @param vertical the analytic context
         */
        startSearch:  function (vertical) {
            var d_dialog = this._d_dialog;
            var parametersMap = {};
            var d_selectedTab = registry.byNode(query('.content', d_dialog.domNode)[0]).selectedChildWidget;

            console.log('AdvancedSearchModel.startSearch', d_selectedTab, d_selectedTab.id.replace('dynamic_', ''), this._groups);
            parametersMap = dynamicUIUtil.generateParameters(d_selectedTab, d_selectedTab.id.replace('dynamic_', ''), this._groups, this._data.properties.concatDelimiter);

            // generate the max rows option
            var d_select = this._d_maxRowsSelect;
            this.enforceConstraints(d_select); 
            var value = d_select.value; 
            
            parametersMap.queryParams.push({
                queryParameter: 'maxRows',
                queryType: 'GENERAL',
                queryValue: '%0'.tokenize(value)
            });
        
            console.log('advanced search query parameters', parametersMap);
            this.controller.submissionFunction({}, vertical, parametersMap, true);
        },

        /**
         * wires up UI controls to user preferences functionality
         */
        setUpPreferences:function () {
            var that = this;

            var restorePreferencesButton = registry.byId('advancedsearch-restore');
            on(restorePreferencesButton, 'click', function (e) {
                lang.hitch(that, resetValues)();
                UserPreferenceUtil.remove(preferenceKey);
            });
        },

        /**
         * closes the advanced search dialog and destroys it
         */
        close: function() {
            this._d_dialog.destroyDescendants();
            this._d_dialog.destroy();
        },

        /**
         * enables the advanced search button on the toolbar
         */
        enableButton:function () {
            var toolbarButton = getToolbarButton();

            if (toolbarButton) {
                toolbarButton.set('disabled', '');
            }
        },

        /**
         * disables the advanced search button on the toolbar
         */
        disableButton:function () {
            var toolbarButton = getToolbarButton();

            if (toolbarButton) {
                toolbarButton.set('disabled', 'disabled');
            }
        }
    });
});
