/**
 * AlertsViewModel is an instantiable object responsible for rendering the AlertsData and intercepting events on it. It also
 * registers facets. Complex events (that contain logic or switch views) are contained in {@link controller.AlertsViewController} while
 * DOM manipulation is done here.  
 * 
 * @class model.AlertsViewModel Renders the AlertsView and handles UI events
 */
define([
    'sna/model/widgets/LoadingSpinner/LoadingSpinner',
    './Locator',
    'dojo/dom-style',
    'dojo/_base/lang',
    'dojox/lang/functional',
    'dojo/_base/array',
    'dijit/registry',
    './widgets/MeasuresTable',
    './viewactions/ObjectModelUtil',
    'dojo/_base/declare',
    'dojo/dom-construct',
    'dojo/text!src/sna/resources/html/models/alertsview.html',
    'dojo/query',
    'dojo/on',
    'dojo/ready'
], function(LoadingSpinner, locator, domStyle, lang, functional, array, registry, MeasuresTable,
		    objectModelUtil, declare, construct, alertsView, query, on, ready) {
    "use strict"; 
    
	var facetTypeName = 'alertsview';
	var that = this; 
	
    /**
     * executed when an alert is clicked on the alerts view (no side-effects)
     * @param row
     */
    var alertClicked = function(row, entities, vertical, submissions) {
        var entity = objectModelUtil.getActionableEntity(entities,  objectModelUtil.getActionableEntityId(row));
        entity.measuresRow = row;
        this.controller.alertClicked(entity, vertical, submissions);
    };

    var getActionableEntities =  function(d_dataGrid, checkedItems) {
        var entities = [];

        array.forEach(checkedItems, function(value, index) {
            var node = d_dataGrid.row(value);
            var actionableEntityId = d_dataGrid.dataGenerator.getActionableEntityId(node.data);
            entities.push(d_dataGrid.dataGenerator.getActionableEntity(actionableEntityId));
        });

        return entities;
    };

	return declare([], {
        /**
         * @constructor creates a new AlertsviewModel
         * @param args object of the form :
         * <pre><code>
         *     {
         *      controller: the {@link controller.AlertsViewController} that will control the model,
         *      d_context: the outer-most container that represents the alerts view
         *     }
         * </pre></code>
         */
		constructor: function( args ) {
			this.controller = args.controller;
			this.d_context = args.d_context;
		},
		
		/**
		 * does cleanup work for when this is view is no longer active
		 */
		cleanupView: function() {
            registry.byNode(this.d_context).destroyDescendants();

            var d_facetContainer = locator.locate().alertsfacetcontainer();
            if(d_facetContainer) {
	            d_facetContainer.collapse();
	            d_facetContainer.hide();
            }
		},
		
        /**
         * renders a data grid with the alerted entities
         * @param sVertical the analytic domain
         * @param alertedEntities object that contains alerted entities, facets, and alerts view plugins
         * @param submissions the advanced search criteria (if any)
         */
         render: function( sVertical, alertedEntities, submissions, showFacetPanel ) {
			var that = this;
            var completedLoadingFlag = false;
//            new LoadingSpinner().target(registry.byId("container")).until(function() { return completedLoadingFlag; }).startSpinners(function() {});
            this._searchParameters = submissions;
            construct.place(alertsView, this.d_context);

            domStyle.set(query('#noalertsmessage')[0], 'display', 'none');

            var oncheck;
            oncheck = function(event, object, d_checkbox) {
                var checked = query('input[type=checkbox][aria-checked="true"]:checked', d_dataGrid.domNode);
                var count = checked.length;
                if(d_checkbox.checked) {
                    count += 1;
                    checked.push(d_checkbox.domNode);
                }
                // get all actionable entities
                var entities = getActionableEntities(d_dataGrid, checked);
                that.controller.updateDispositionItemsList(entities, 'alerts');
            };

             var d_dataGrid;
             var grid = new MeasuresTable({
                 data: alertedEntities,
                 oncheck: oncheck,
                 options: {
                     useCheckboxes: true
                 }
             });
             
             var filterQueryFunction = this.controller.getFilterQueryFunction();
             d_dataGrid = grid.render('MEASURES', query('#grid')[0], false, {
                 onclick: function(row) {
                     lang.hitch(that, alertClicked, row.data, alertedEntities.actionableEntities, sVertical, submissions)();
                 }
             }, {
                 enableSelection: true
             }, 
                 filterQueryFunction
             );
             this._d_dataGrid = d_dataGrid;
             this._printData = alertedEntities;
             
             var d_facetContainer = locator.locate().alertsfacetcontainer();
             // hide the facets panel if there is no facet data
             if(showFacetPanel === 'n') {
            	 d_facetContainer.hide();
             }
             else if(alertedEntities.facets.length === 0) {
            	 d_facetContainer.collapse(); 
             }
             else {
            	 d_facetContainer.expand(); 
             }
             
             ready(function() {
                 that.controller.startDispositionController(sVertical, function() {
                     // filter by view first
                     that.controller.updateDispositionItemsList([], 'alerts');
                 });

                 that.controller.startExternalAppController(function() {
                	 that.controller.updateExternalAppsList([], 'alerts');
                 });
                 completedLoadingFlag = true;
             });
        },

        refreshDataGrid:function () {
            if (this._d_dataGrid) {
                this._d_dataGrid.refresh();
            }
        },
		
		dispositionCompleted: function() {

		}, 
		
		getPrintData: function() { 
			return {
                tables: this._printData
            };
		},
		
		getAdvancedSearchParameters: function() {
			return this._searchParameters;
		},

        /**
         * gets a list of selected (checked) actionable entities in the data grid
         */
        getSelectedAlerts: function() {
            console.log('alertsviewmodel.getSelectedAlerts', this._d_dataGrid);
            return [];
        },

        /**
         * searches the datagrid columns based on a column name and returns the index
         * @param {string} columnIdentifier the column name
         * @return {Number} the column index
         */
        getColumnIndexFromColumnIdentifier:function (columnIdentifier) {
            var index = -1;

            var that = this;

            if (this.isDGridReady()) {
                that._d_dataGrid._columnsMap = that._d_dataGrid._columnsMap || {};

                if(that._d_dataGrid._columnsMap[columnIdentifier]) {
                    return that._d_dataGrid._columnsMap[columnIdentifier];
                }

                functional.forIn(that._d_dataGrid.columns, function (column, count) {
                    if (column.field === columnIdentifier) {
                        index = count;
                        that._d_dataGrid._columnsMap[columnIdentifier] = count;
                    }
                });
            }

            return index;
        },
		
		isDGridReady: function() {
			if( this._d_dataGrid ) {
				return true;
			}
			
			return false;
		},

        destroyDataGrid: function() {
            if(this._d_dataGrid) {
                if(this._d_dataGrid.destroy) {
                    this._d_dataGrid.destroy();
                }
                this._d_dataGrid = undefined;
            }
        },

        /**
         * removes all of the checked rows in the alerts data grid.
         */
        removeCheckedRows: function() {
            var d_dataGrid = this._d_dataGrid;
            var checkedElements = query('input[type=checkbox][aria-checked="true"]:checked', d_dataGrid.domNode);

            console.log('AlertsViewModel.removeCheckedRows', d_dataGrid);

            array.forEach(checkedElements, function(value) {
                var d_row = d_dataGrid.row(value);
                console.log('removing ',d_row);
                d_dataGrid.store.remove(d_row.data[0]);
            });
        }
	});
});
