/**
 * @class model.viewactions.DynamicUIUtil
 * provides utility functions to manipulate dynamically generated UIs (see {@link model.viewactions.DynamicUIRenderer})
 *
 */
define([
    'sna/util/DateUtil',
    'dojo/_base/array'
], function(dateUtil, array) {
    return {
        /**
         * pulls data from forms and returns a map representation
         *
         * @param d_selectedTab the ContentPane that was selected
         * @param formId the id of the section that will be submitted
         * @param groups a list of ContentPanes that represent individual parameters on the form
         * @return {Object} an object of the form:
         * <pre><code>
         *     {
         *          queryParams: [
         *              ...
         *              {
         *                  queryType: formId,
         *                  queryParameter: the group this parameter belongs to
         *                  queryValue: the value from the form element, or the concatenated value in the case of multiple
         *              }
         *          ]
         *     }
         *</pre></code>
         */
        generateParameters: function(d_selectedTab, formId, groups, concatDelimiter) {
            var delimiter = '|';
            if(!concatDelimiter) {
                console.warn('DynamicUIUtil.generateParameters: concatDelimiter was undefined, assuming pipe character');
            }
            else {
                delimiter = concatDelimiter;
            }

            var parameters = [];

            array.forEach(groups, function(group, index) {
                var parameter = {
                    queryType: group.getParent().get('data-form-id'),
                    queryParameter: group.id
                };

                if(parameter.queryType !== d_selectedTab.get('id').replace('dynamic_', '')) {
                    return; // don't submit it if these queries aren't a part of the current tab
                }

                var children = group.getChildren();

                array.forEach(children, function(child, childIndex) {
                    array.forEach(child.getChildren(), function(childElement, childElementIndex) {
                        if(childElement.declaredClass === 'dijit.form.TextBox' || childElement.declaredClass === 'dijit.form.SimpleTextarea') {
                            if(parameter.queryValue) {
                                parameter.queryValue += delimiter + childElement.value;
                            }
                            else {
                                parameter.queryValue = childElement.value;
                            }
                        }
                        else if(childElement.declaredClass === 'sas.sna.model.widgets.DateTextBox' || childElement.declaredClass === 'dijit.form.DateTextBox') {

                            if (childElement.value !== null) {
                                var time = childElement.value.getTime();
                            } else { //if the date is deleted set the variable to undefined
                                var time = undefined;
                            }

                            var actualValue = '';

                            if(!isNaN(time)) { //this is true when the field has a valid date in it.
                                actualValue = dateUtil.toSasDate(childElement.value);
                            }

                            if(parameter.queryValue) {
                                parameter.queryValue += delimiter + actualValue;
                            }
                            else {
                                parameter.queryValue = actualValue;
                            }
                        }
                        else if (childElement.declaredClass === 'dijit.form.RadioButton' || childElement.declaredClass === 'dijit.form.CheckBox') {
                            if(childElement.checked) {
                                if(parameter.queryValue) {
                                    parameter.queryValue += delimiter + childElement.value;
                                }
                                else {
                                    parameter.queryValue = childElement.value;
                                }
                            }
                        }
                        else if (childElement.declaredClass === 'sas.sna.model.widgets.PreferencesMultiSelect') {
                            var element = childElement.getChildren();

                            array.forEach(element, function(value) {
                                if(value.selected) {
                                    if(parameter.queryValue) {
                                        parameter.queryValue += delimiter + value.value;
                                    }
                                    else {
                                        parameter.queryValue = value.value;
                                    }
                                }
                            });

                            //alert(parameter.queryValue);
                        }
                        else if (childElement.declaredClass === 'dijit.form.Select') {
                            parameter.queryValue = childElement.value;
                        }
                        else {
                            console.warn('didnt recognize ' + childElement.declaredClass, childElement);
                            parameter.queryValue = childElement.value;
                        }
                    });
                });

                if(typeof(parameter.queryValue) === 'undefined') {
                    parameter.queryValue = '';
                }

                parameters.push(parameter);
            });

            return {
                queryParams: parameters
            };
        }
    };
});
