/**
 * @class model.viewactions.GraphBuilder handles construction of graph generators (pie, line, scatter, etc.)
 * Used by ChartBuilder
 */
define([
    'dojo/number',
    'sna/util/I18nUtil',
    'dojo/dom-style',
    'dojo/_base/lang',
    'dojox/charting/widget/Legend',
    'dojox/charting/action2d/Magnify',
    'dojox/charting/action2d/Tooltip',
    'dojox/charting/action2d/MoveSlice',
    'dojox/lang/functional',
    'dojo/dom-construct',
    'dijit/layout/BorderContainer',
    'dijit/layout/ContentPane',
    'dojo/_base/declare',
    'dojox/charting/Chart',
    'dojox/charting/themes/Claro',
    'dojox/charting/plot2d/Scatter',
    'dojox/charting/plot2d/Lines',
    'dojox/charting/plot2d/Pie',
    'dojox/charting/plot2d/Bubble',
    'dojox/charting/plot2d/ClusteredColumns',
    'dojox/charting/plot2d/Markers',
    'dojox/charting/axis2d/Default',
    'dojox/charting/plot2d/Grid'
], function(number, I18nUtil, style, lang, Legend, Magnify, Tooltip, MoveSlice, functional, domConstruct, BorderContainer, ContentPane, declare, Chart, Shrooms) {
    var getYAxis = function (subpanel) {
        var axisObject = {
            dropLabels:false,
            minorTicks:false,
            vertical:true
        };

        if (typeof subpanel.min !== 'undefined' && typeof subpanel.max !== 'undefined') {
            if (Math.abs(subpanel.max - subpanel.min) <= 1e-6) {
                axisObject.max = subpanel.max + subpanel.max / 2;
                axisObject.min = subpanel.min - subpanel.min / 2;
            }
            else if (subpanel.max < 500 && subpanel.min >= 0) {
                axisObject.max = subpanel.max;
                axisObject.min = 0;
            }
            else {
                if(subpanel.chartType === 'bar') {
                    axisObject.min = subpanel.min - 1;
                }
            }
        }

        // always show ticks for the lower and upper values on the chart
        axisObject.fixLower = 'major';
        axisObject.fixUpper = 'major';

        return axisObject;
    };

    return declare(null, {
        getChartRenderer:  function(subpanel, d_target, subpanelCount, chartType, options) {
            // per S0943765	SNA graph series get displayed in random order we now sort in order of series number
            // this is because some clients may want to display series in a specific order and since 2.3
            // they are in an unordered map
            // also only do this once instead of every time a chart is rendered
            var keysToSort = Object.keys(subpanel.series);
            var sortedSeriesKeys = keysToSort.sort(function (a, b) {
                var aValue = 0;
                var bValue = 0;

                if (!a.startsWith('series')) {
                    aValue = 0;
                }
                else {
                    aValue = number.parse(a.replace('series', '').replace('_name', ''));
                }

                if (!b.startsWith('series')) {
                    bValue = 0;
                }
                else {
                    bValue = number.parse(b.replace('series', '').replace('_name', ''));
                }

                return aValue - bValue;
            });

            return function() {
                // cleanup of previous render
                d_target.destroyDescendants();
                var chartOptions = lang.mixin({}, options);

                var d_container = new BorderContainer({
                    style: 'width: 100%; height: 100%;',
                    liveSplitters: false,
                    gutters: false
                });

                var formattedId = subpanel.formattedId || subpanel.chartId;
                if(subpanel.formattedId && subpanel.localizeFormattedId) {
                    formattedId = I18nUtil.get(subpanel.formattedId);
                }

                var d_titlePane = new ContentPane({
                    content: '<header>%0</header>'.tokenize(formattedId),
                    region: 'top',
                    style: 'padding: 0'
                });
                d_container.addChild(d_titlePane);
                var titleHeight = 25;

                var d_chartPane = new ContentPane({
                    style: 'width: 100%; padding: 0 !important; ',
                    region: 'center'
                });
                d_container.addChild(d_chartPane);

                // note: for some reason if you use the contentpane domNode property when creating the legend,
                // it complains about a duplicate id. I'm not sure why...
                var legendHeight = 10;
                if(typeof(chartOptions.disableLegend) === 'undefined' || chartOptions.disableLegend === false) {
                    legendHeight = 30;
                    var legendContent = domConstruct.toDom('<div></div>');

                    var d_legendPane = new ContentPane({
                        region: 'bottom',
                        content: legendContent,
                        style: 'padding: 0 !important;'
                    });
                    d_container.addChild(d_legendPane);
                }

                // these numbers are derived totally from experimentation
                var columnGap = 30 - 2 * subpanel.series.category_field.length - 2 * subpanelCount;
                if(columnGap < 5) {
                    columnGap = 5;
                }

                var getStrokeColor = function() {
                    if(options && options.getStrokeColor) {
                        return options.getStrokeColor(subpanel.series.category_field, subpanel.categoryColors);
                    }

                    return undefined;
                };

                var myTheme =  Shrooms;

                // some predefined themes can be passed in externally, such as the 'white' theme which
                // is is used for printing
                if(subpanel.chartTheme && subpanel.chartTheme === 'white') {
                    myTheme.fill = myTheme.plotarea.fill = myTheme.chart.fill = "white";
                    myTheme.axis.stroke.color = myTheme.axis.tick.fontColor = 'black';
                }
                else if (subpanel.chartTheme && subpanel.chartTheme === 'blueSteel') {
                    myTheme.fill = myTheme.plotarea.fill = myTheme.chart.fill = "#454852";
                    myTheme.axis.stroke.color = myTheme.axis.tick.fontColor = 'white';
                }

                // graphs with < 25 observations need to show their tick labels. Otherwise,
                // hide them for readability
                var majorTickStep = 1;
                functional.forIn(subpanel.series, function(item) {
                    if(item.length && item.length > 25) { // completely arbitrary
                        majorTickStep = undefined;
                    }
                });

                var xAxis = {
                    minorTicks: false,
                    labelFunc: function(text, value, precision) {
                        if(precision === 0) {
                            var label = subpanel.series.category_field[value - 1];
                            // could be undefined due to scaling issues
                            if(typeof(label) !== 'undefined' && label.length > 10) {
                                label = label.substring(0, 10) + '...';
                            }

                            return label;
                        }
                    }
                };

                if(typeof(majorTickStep) !== 'undefined') {
                    xAxis.majorTickStep = majorTickStep;
                }

                // lowest value in series doesn't work out
                // building of chart data
                var chart = new Chart(d_chartPane.domNode)
                    .setTheme(myTheme)
                    .addAxis('x', xAxis)
                    .addAxis('y', getYAxis(subpanel))
                    .addPlot("default", { type:chartType, markers:true, gap: columnGap, stroke: getStrokeColor()});


                // TODO what to do when there are no real labels other than series1_name etc.?
                // we iterate through sortedSeriesKeys instead of the series map directly because they need
                // to have an order to them
                functional.forIn(sortedSeriesKeys, function(key) {
                    var value = subpanel.series[key];

                    if(key.startsWith('series')) {
                        var seriesColor = {};
                        var seriesColorsEnabled = !(options && options.disableSeriesColors);

                        if(subpanel.seriesColors && subpanel.seriesColors[key] && seriesColorsEnabled) {
                            seriesColor = {
                                stroke: subpanel.seriesColors[key].replace('0x', '#'),
                                fill: subpanel.seriesColors[key].replace('0x', '#')
                            };
                        }

                        if(options && options.seriesModifier) {
                            value = options.seriesModifier(value, subpanel.series.category_field, subpanel.categoryColors);
                        }

                        if(subpanel.seriesLabels && subpanel.seriesLabels[key]) {
                            chart.addSeries(subpanel.seriesLabels[key], value, seriesColor);
                        }
                        else {
                            chart.addSeries(key, value, seriesColor);
                        }
                    }
                });

                // Create the tooltip
                var tip = new Tooltip(chart,"default", {
                    text: function(element) {
                        return '<div>%0</div><div><b>%1:</b> %2</div>'.tokenize(subpanel.series.category_field[element.index], element.run.name, element.y);
                    }
                });

                // render chart
                chart.render();
                d_target.addChild(d_container);
                d_target.startup();

                if(typeof(chartOptions.disableLegend) === 'undefined' || chartOptions.disableLegend === false) {
                    new Legend({chart: chart, horizontal: true}, legendContent);
                }

                if(chartOptions.pluginsCreator) {
                    chartOptions.pluginsCreator(chart);
                }

                chart.resize(style.get(d_chartPane.domNode, 'width'), style.get(d_target.domNode, 'height') - legendHeight - titleHeight);
                d_target.resize();
            };
        },

        getPieRenderer:function (subpanel, d_target, subpanelCount) {
            return this.getChartRenderer(subpanel, d_target, subpanelCount, 'Pie', {
                disableSeriesColors: true,
                disableLegend:true,
                pluginsCreator:function (chart) {
                    new MoveSlice(chart, "default");
                },
                /**
                 * have to modify the series for pie charts explicitly because users can supply their own slice colors
                 */
                seriesModifier: function(series, categoryField, categoryColors) {
                    if(!(categoryColors && Object.keys(categoryColors).length > 0 && categoryField && categoryField.length === Object.keys(categoryColors).length)) {
                        return series;
                    }

                    var newSeries = [];

                    functional.forIn(series, function(value, index) {
                        // series starts out as an array of numbers like [1, 2, 3, 4]
                        // but pie charts have to convert this to [{y: 1, color: "foo"}, ...]
                        // if category colors have been defined
                        newSeries[index] = {
                            y: value,
                            color: categoryColors[categoryField[index]].replace('0x', '#')
                        };
                    });

                    return newSeries;
                },
                getStrokeColor: function(categoryField, categoryColors) {
                    // use black if user provides their own colors
                    if(categoryColors && Object.keys(categoryColors).length > 0 && categoryField && categoryField.length === Object.keys(categoryColors).length) {
                        return '#000000';
                    }
                    else {
                        return '#ffffff';
                    }
                }
            });
        }
    });
});