/**
 * @name model.viewactions.ObjectModelUtil
 * @class provides utility methods to map the object model correctly for consumers (such as the data grid)
 */
define([
    'dojo/dom-construct',
    'dojo/_base/lang',
    'dojo/query',
    'dojo/dom-geometry',
    'dojo/dom-style',
    'dojo/on',
    'dojo/number',
    'dojo/store/util/QueryResults',
    '../widgets/KPIIndicator',
    'dojo/_base/declare',
    'dojo/_base/array',
    'dojo/store/Memory',
    'dojo/dom-construct',
    'dojo/dom',
    'sna/model/widgets/AlertSeverityIndicator',
    'dojo/dom-attr',
    'dojo/NodeList-traverse'
], function(domConstruct, lang, query, domGeometry, domStyle, on, number, QueryResults, KPIIndicator, declare, array, Memory, construct, dom, AlertSeverityIndicator, domAttr) {
    var _getActionableEntity = function(entities, id) {
        return array.filter(entities, function(item) {
            return item.actionableEntityId === id;
        })[0];
    };

    var _getRowIdentity = function (row) {
        return row[0][0];
    };

    var _tableHasActionableEntities = function(tableData) {
        if(tableData.values.length > 0 && tableData.values[0].length > 0) {
            return tableData.values[0][0].length === 1;
        }

        return false;
    };

    var _unescapeValue = function(value) {
        if(value.indexOf('\\u') !== -1 || value.indexOf('#') !== -1 || value.indexOf('&') !== -1) {
            var unescaped = JSON.parse('"%0"'.tokenize(value.split('\\').join('\\\\')));
            var element = domConstruct.toDom('<span>%0</span>'.tokenize(unescaped));

            return element.innerText || element.textContent
        }

        return value;
    };

    var _getRawValue = function(value) {
        var valueToUse = value[0];

        if(value[0] === "") {
            valueToUse = value[1];
        }

        return _unescapeValue(valueToUse);
    };

    var _getFormattedValue = function(value) {
        if(value.length === 1) {
            return ''; // not a valid row value
        }

        if(value[1] === null) {
            return null;
        }

        return _unescapeValue(value[1]);
    };

    var getCellRenderer = function(columnObject, column, columnIndex, other, style) {
        return function(object, value, node, options) {
            var columnData = object[columnIndex];
            var rawValue = _getRawValue(columnData);
            var formattedValue = _getFormattedValue(columnData);

            var decorateNode = function(node) {
                if(columnIndex === 1) {
                    if(other && other.actionableEntities) {
                        domAttr.set(node, 'data-id', _getActionableEntity(other.actionableEntities, _getRowIdentity(object)).actionableEntityId);
                    }
                }
            };

            var columnStyle  = columnData[2] + style;

            // just a regular column
            if(columnData.length === 3) {
                var newNode = construct.toDom('<div style="%0">'.tokenize(columnStyle) + formattedValue + '</div>');
                decorateNode(newNode);

                return newNode;
            }

            // special column, format appropriately
            var special = columnData[3];

            var component;

            if(special.type === 'severity') {
                component = new AlertSeverityIndicator({value: columnData[0], max: special.max, min: special.min, median: special.median}).placeAt(node);
            }
            else if (special.type === 'kpi') {
                component = new KPIIndicator({target: special.target, score: special.score}).placeAt(node);
            }
            else if (special.url) {
                var urlNode = construct.toDom('<div style="%0"><a href="%1" target="_blank">%2</a></div>'.tokenize(columnStyle, special.url, formattedValue));
                on(urlNode, 'click', function(event) {
                    event.stopPropagation();
                });
                construct.place(urlNode, node) ;
            }
            else if (special.type === 'image') {
                var imageNode = construct.toDom('<div style="%0"><img src="%1"></div>'.tokenize(columnStyle, special.src));
                construct.place(imageNode, node);
            }

            if(component) {
                // TODO find a better way to do this because this is not the most elegant solution
                this.grid.itemsToDestroy = this.grid.itemsToDestroy || [];
                this.grid.itemsToDestroy.push(component);
                component.placeAt(node);
            }

            decorateNode(node);
        };
    };

    var _columnIsHidden = function(column) {
        return _getRawValue(column) === null ||  _getRawValue(column).startsWith('__') || (column.length > 3 && typeof(column[3].hidden) !== 'undefined' && column[3].hidden === true);
    };

    return {
        /**
         * returns a function that will generate data in a correct format for rendering to a data grid
         * @function
         * @param table the table name to use
         * @param data an object of the structure:
         * <pre><code>
         * {
         *      ...
         *      tables: {
         *          "table name": {
         *              columns: [..],
         *              values: [..]
         *          }
         *      }
         * }
         * </code></pre>
         * @return Object object of the structure
         * <pre><code>
         * {
         *      columns, (function, returns an array of column objects)
         *      rowsStore, (function, returns a store that has all of the row data for a table.
         *         with actionable entity ids but bad for everone else
         * }
         * </code></pre>
         */
        getDataGenerator: function(table, data) {
            var tableData = data.tables[table];
            var that = this;

            var increment = 0;
            if(_tableHasActionableEntities(tableData)) {
                increment = 1;
            }

            /**
             * this custom Memory store is used because of our object model, stores assume an identity property but each row in our object
             * model is an array (of arrays).
             * @type {*}
             */
            var CustomMemory = declare([Memory], {
                sortFunction: function(a, b, column, descending) {
                    var columnIndex = 0;

                    array.forEach(tableData.columns, function(value, index) {
                        if(that.getRawColumnValue(value) === column) {
                            columnIndex = index;
                            return;
                        }
                    });

                    var first = that.getRawValue(a[columnIndex + increment]);
                    var second = that.getRawValue(b[columnIndex + increment]);

                    if(isNumber(first) && isNumber(second)) {
                        if(descending) {
                            return number.parse(second) - number.parse(first);
                        }
                        else {
                            return number.parse(first) - number.parse(second);
                        }
                    }

                    if(descending) {
                        return second.localeCompare(first);
                    }
                    else {
                        return first.localeCompare(second);
                    }
                },

                setData: function(data){
                    this.data = data;
                    this.index = {};
                    for(var i = 0, l = data.length; i < l; i++){
                        this.index[this.getIdentity(data[i])] = i;
                    }
                },
                getIdentity: function(object) {
                    return _getRowIdentity(object);
                },

                query:function(query,options){
                    var sortFunction = this.sortFunction;

                    if(options.sort!=undefined){
                        if(options.custom) {
                            return this.inherited(arguments);
                        }
                        else if(options.sort.length>0){
                            var sort = options.sort[0];

                            console.log(options);
                            options.custom = {
                                attribute: sort.attribute,
                                descending: sort.descending
                            };

                            options.sort = function(a, b) {
                                return sortFunction(a, b, sort.attribute, typeof(options.custom.descending) !== 'undefined' && options.custom.descending === true);
                            };

                            return this.inherited(arguments);
                        }
                    }

                    return this.inherited(arguments);
                }
            });

            var rowsStore = new CustomMemory({
                data:tableData.values
            });

            return {
                columns: function() {
                    var index = 0;
                    var mappedColumns = array.map(tableData.columns, function(column) {
                        var newColumn = (function(i) {
                            var columnObject = {};
                            lang.mixin(columnObject, {
                                field: _getRawValue(column),
                                label: _getFormattedValue(column),
                                get: function(element) {
                                    // get the formatted column value
                                    return element[i+increment][1];
                                },
                                renderCell: getCellRenderer(columnObject, column, i+increment, {actionableEntities: data.actionableEntities}, column[2]),
                                renderHeaderCell: function(node) {
                                    var columnStyle = '';
                                    if(column[3] && column[3].headerStyle) {
                                        columnStyle = column[3].headerStyle;
                                    }

                                    if(column[3] && column[3].minWidth && typeof(this.alreadySized) === 'undefined') {
                                        this.grid.styleColumn(this.id, 'width: %0px;'.tokenize(column[3].minWidth));
                                        this.alreadySized = true;
                                    }
                                    else {
                                        this.grid.styleColumn(this.id, 'width: %0px;'.tokenize(200));
                                        this.alreadySized = true;
                                    }

                                    return construct.toDom('<div style="%0">%1</div>'.tokenize(columnStyle, column[1]));
                                },
                                hidden: _columnIsHidden(column),
                                unhidable: _columnIsHidden(column)
                            });

                            return columnObject;
                        }(index));
                        index += 1;

                        return newColumn;
                    });

                    return mappedColumns;
                },
                rows: function() {
                    return tableData.values;
                },
                rowsStore: function() {
                    return rowsStore;
                },
                getActionableEntity: function(id) {
                    return _getActionableEntity(data.actionableEntities, id);
                },
                getActionableEntityId: function (row) {
                    return _getRowIdentity(row);
                },
                getRawTableData: function() {
                    return tableData;
                }
            };
        },

        safeUnescapeValue: function(value) {
            return _unescapeValue(value);
        },

        /**
         * gets an actionable entity matching a specific id
         * @param entities array of actionable entity objects
         * @param id the id to match
         * @return {Object}
         */
        getActionableEntity:  function(entities, id) {
            return _getActionableEntity(entities, id);
        },

        /**
         * given an array that represents a row of data, gets the actionable entity id it is tied to
         * @param row
         */
        getActionableEntityId: function(row) {
            return _getRowIdentity(row);
        },

        /**
         * given an array representing a column, returns the column name
         * @param column object of the structure:
         * <pre><code>
         * [
         *      raw column value,
         *      formatted value,
         *      css styling
         * ]
         * </code></pre>
         * @return {string}
         */
        getRawColumnValue: function(column) {
            if(column[0] === "") {
                return column[1];
            }
            else {
                return column[0];
            }
        },

        /**
         * given a value representing an element in a row, returns the raw unformatted value
         * @param value of the structure:
         * <pre><code>
         * [
         *      rawValue,
         *      formattedValue,
         *      css styling,
         *      other properties (optional)
         * ]
         * </code></pre>
         * @return {string}
         */
        getRawValue: function(value) {
            return _getRawValue(value);
        },

        /**
         * given a value representing an element in a row, returns the format
         * @param value of the structure:
         * <pre><code>
         * [
         *      rawValue,
         *      formattedValue,
         *      css styling,
         *      other properties (optional)
         * ]
         * </code></pre>
         * @return {string}
         */
        getFormattedValue: function(value) {
            return _getFormattedValue(value);
        },

        /**
         * returns special properties of a column (such as type, style, etc.)
         *
         * @param column
         * @return {*|Object}
         */
        getSpecialColumnProperties: function(column) {
            return column[3] || {};
        },

        /**
         * if a row has an actionable entity id, it comes before the other columns
         * @param row
         */
        getColumnIndex: function(row, index) {
            if(row[0].length === 1) {
                return (1 + number.parse(index));
            }

            return (0 + number.parse(index));
        }
    };
});