/**
 * @class 
 * Manages a group of dijit checkbox components, so their behavior can be managed together
 * 
 * Provides functions that allow User Preferences to be set
 */
define([
    'sna/model/viewactions/ObjectModelUtil',
    'dijit/form/CheckBox',
    './PreferencesDefaultRendererMixin',
    './Table',
    'dojo/on',
    'dojo/dom-construct',
    'dijit/_WidgetBase',
    'dojo/_base/declare',
    'dojox/lang/functional',
    'dojo/_base/lang',
    'dijit/layout/ContentPane'
], function(ObjectModelUtil, CheckBox, PreferencesDefaultRendererMixin, Table, on, domConstruct, _WidgetBase, declare, functional, lang, ContentPane ) {
	
	/**
	 * set all checkboxes to unchecked
	 */
	var clearAll = function() {
		// iterate over this.checkedItems and set all to false
        var that = this;
		functional.forIn(this.checkedItems, function(item, key) {
            that.checkedItems[key] = false;
        });

		functional.forIn(this.checkboxes, function( cb, index ) {
			cb.set('value', false);
		});

        this.checkedItemCount = 0;
	};

	/**
	 * set the checkbox with the passed-in name to be checked
	 */
	var setCheckbox = function( v ) {
		this.checkedItems[v] = true;
        this.checkedItemCount += 1;
		
		functional.forIn(this.checkboxes, function( cb, index ) {
			if( cb.get('name') === v ) {
				cb.set('value', true);
			}
		});
	};
	
	var renderCheckboxes = function() {
		var that = this;

        this.checkGroupTable = new Table({
            maxColumns: this.maxColumns
        });
        var groupRow = this.checkGroupTable.addRow(0);

		functional.forIn(this.checkboxes, function( cb, index ) {
            that.checkedItems[cb.key] = false;

            // when checkbox groups get updated with new entries, renderCheckboxes is called again
            // So if we don't hold a reference to the old event handler, a checkbox can have multiple
            // event handlers attached to it
            if(!cb.onSignal) {
                var onSignal = on(cb, 'click', function() {
                    console.info('click registered');

                    if(cb.checked) {
                        that.checkedItems[cb.key] = true;
                        that.checkedItemCount += 1;
                    }
                    else {
                        that.checkedItems[cb.key] = false;
                        that.checkedItemCount -= 1;
                    }
                });
                cb.onSignal = onSignal;
            }
            else {
                console.info('skipping on event');
            }

            var elementTable = new Table({
                rows: 2,
                columns: 2
            });

            var labelText = cb.formattedKey || '';
            var label = domConstruct.toDom('<label for="">' + labelText + '</label>');
            elementTable.placeElement(cb.domNode, label, that.labelPlacement || 'right');
            groupRow = that.checkGroupTable.addColumn(groupRow, elementTable.getTable().domNode).row;
		});

        domConstruct.place(this.checkGroupTable.getTable().domNode, this.contentPane.domNode);

		// see comment above the add method to see why we did this
		this.add = undefined;

        /**
         * track the escaped values so that it is quick to compare them when filtering
         */
        this._escapedValues = {};

        functional.forIn(this.checkboxes, function(checkbox) {
            that._escapedValues[ObjectModelUtil.safeUnescapeValue(checkbox.key)] = checkbox.key;
        });
        // --
	};

	var getCheckbox = function( value ) {
		var foundCheckboxIndex = -1;
		
		functional.forIn(this.checkboxes, function( cb, index ) {
			if( cb.value === value ) {
				foundCheckboxIndex = index;
			}
		});
		
		return foundCheckboxIndex;
	};
	
    return declare([_WidgetBase, PreferencesDefaultRendererMixin], {
        constructor: function(args) {
            this.id = args.id;
            this.name = args.key;
            this.key = args.key;
            this.formattedKey = args.formattedKey;
            this.maxColumns = args.maxColumns || 0;
            this.checkboxes = [];
            this.checkedItems = {};
            this.checkedItemCount = 0;
            this.labelPlacement = args.labelPlacement;
            args.cssClass = 'dynamic-widget';
            args.disableGrid = true; // don't use table to render the checkbox label
            this.inherited(arguments);
        },

        createCheckbox: function(checkboxJsonObject) {
            return new CheckBox({
                name: checkboxJsonObject.name,
                value: checkboxJsonObject.value,
                checked: (checkboxJsonObject.checked || false),
                formattedKey: (checkboxJsonObject.formattedKey || checkboxJsonObject.value),
                key: (checkboxJsonObject.key || checkboxJsonObject.value),
                onChange: (checkboxJsonObject.onChangeCallback || function(){})
            });
        },

        /**
         * Add the passed-in checkbox to the group
         * NOTE: this method is not meant to be invoked after render (this can be a future enhancement to this component)
         */
        addCheckbox: function( checkboxJsonObject ) {
            this.checkboxes.push(this.createCheckbox(checkboxJsonObject));
        },
		
        /**
         * set a field on all CheckBox components managed in this group
         */
		setField: function( field, values ) {
			var that = this;
			lang.hitch(that, clearAll)();
			functional.forIn(values, function( v, index ) {
				lang.hitch(that, setCheckbox, v)();
			});
		},
		
		/**
         * get a field on this component
         */
        getField: function( field ) {
			return this[field];
        },
		
		/**
		 * Return an object representing the state of this component to save in User Preferences 
		 */
		getPreferenceValue: function() {
			var justCheckboxValues = [];
			
			functional.forIn(this.checkboxes, function( value, index ) {
				if( value.get('checked') === true ) {
				    justCheckboxValues.push(value.get('name'));
				}
			});
			
			return {
				'value': justCheckboxValues
			};
		},
		
		/**
		 * Clear the value of all the CheckBoxes components in this group 
		 */
		clear: function() {
			lang.hitch(this, clearAll)();
		},

		/**
         * Add this component to a content pane, which is then attatched to the passed-in parent node
         */
		render: function( d_parent) {
            this.inherited(arguments);

			lang.hitch(this, renderCheckboxes)();
		},
		
		/**
		 * Defines a function to be used to determine if the passed-in value is within the bounds of the this components current value
		 */
		computeFilterLogic: function( value ) {
            var result = true;

            var blankIsFalse = false;
            if( this.snamode === 'ASPECT' ) {
                blankIsFalse = true;
            }

            var noItemsChecked = (this.checkedItemCount === 0);

            if( (blankIsFalse === true) && (noItemsChecked === true) ) {
                result = false;
            }
            else {
                // use an array of booleans
                result = (this.checkedItems[this._escapedValues[value]] === true) || (noItemsChecked === true);
            }

            return result;
		},
		
		updateValue: function( facetJsonObject, onChangeCallback ) {
			var that = this;
			
			this.checkGroupTable.destroy();
			this.contentPane.destroyDescendants();

			var newCheckboxValues = [];

			// create individual Checkbox objects from each string label and add them to group
			functional.forIn( facetJsonObject.values, function( value, index ) {
				var alreadyExistingCheckbox = lang.hitch(that, getCheckbox, value)();
				if(alreadyExistingCheckbox === -1) {
					that.addCheckbox({
						name: value,
						value: value,
						formattedKey: value,
						onChangeCallback: onChangeCallback
					});
				}
                else {
                    that.checkboxes[index] = that.createCheckbox({
                        name: value,
                        value: value,
                        formattedKey: value,
                        onChangeCallback: onChangeCallback
                    });
                }

				newCheckboxValues.push(value);
			});
			
			lang.hitch(this, renderCheckboxes)();
		}
    });
});